@@ -49,16 +49,30 @@ G_DEFINE_TYPE (SeahorsePGPKey, seahorse_pgp_key, SEAHORSE_TYPE_KEY);
  */
 
 static gchar*
-convert_string (const gchar *str)
+convert_string (const gchar *str, gboolean escape)
 {
+    gchar *t, *ret;
+    
     if (!str)
         return NULL;
     
     /* If not utf8 valid, assume latin 1 */
     if (!g_utf8_validate (str, -1, NULL))
-        return g_convert (str, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+    {
+        ret = g_convert (str, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+        if (escape) {
+            t = ret;
+            ret = g_markup_escape_text (t, -1);
+            g_free (t);
+        }
+        
+        return ret;
+    }
+
+    if (escape)
+        return g_markup_escape_text (str, -1);
     else
-        return g_strdup (str);    
+        return g_strdup (str);
 }
 
 static gchar* 
@@ -93,35 +107,20 @@ static SeahorseValidity
 calc_validity (SeahorsePGPKey *pkey)
 {
     g_return_val_if_fail (pkey->pubkey, SEAHORSE_VALIDITY_UNKNOWN);
+    g_return_val_if_fail (pkey->pubkey->uids, SEAHORSE_VALIDITY_UNKNOWN);
 
     if (pkey->pubkey->revoked)
         return SEAHORSE_VALIDITY_REVOKED;
     if (pkey->pubkey->disabled)
         return SEAHORSE_VALIDITY_DISABLED;
-    if (pkey->pubkey->uids->validity <= SEAHORSE_VALIDITY_UNKNOWN)
-        return SEAHORSE_VALIDITY_UNKNOWN;
-    return pkey->pubkey->uids->validity;    
+    return seahorse_validity_from_gpgme (pkey->pubkey->uids->validity);
 }
 
 static SeahorseValidity 
 calc_trust (SeahorsePGPKey *pkey)
 {
     g_return_val_if_fail (pkey->pubkey, SEAHORSE_VALIDITY_UNKNOWN);
-
-    switch (pkey->pubkey->owner_trust) {
-    case GPGME_VALIDITY_NEVER:
-        return SEAHORSE_VALIDITY_NEVER;
-    case GPGME_VALIDITY_MARGINAL:
-        return SEAHORSE_VALIDITY_MARGINAL;
-    case GPGME_VALIDITY_FULL:
-        return SEAHORSE_VALIDITY_FULL;
-    case GPGME_VALIDITY_ULTIMATE:
-        return SEAHORSE_VALIDITY_ULTIMATE;
-    case GPGME_VALIDITY_UNDEFINED:
-    case GPGME_VALIDITY_UNKNOWN:
-    default:
-        return SEAHORSE_VALIDITY_UNKNOWN;
-    }
+    return seahorse_validity_from_gpgme (pkey->pubkey->owner_trust);
 }
 
 static void
@@ -138,12 +137,16 @@ changed_key (SeahorsePGPKey *pkey)
         skey->etype = SKEY_ETYPE_NONE;
         skey->loaded = SKEY_INFO_NONE;
         skey->flags = SKEY_FLAG_DISABLED;
+        skey->keydesc = _("Invalid");
+        skey->rawid = NULL;
         
     } else {
     
         /* The key id */
-        if (pkey->pubkey->subkeys)
+        if (pkey->pubkey->subkeys) {
             skey->keyid = seahorse_pgp_key_get_cannonical_id (pkey->pubkey->subkeys->keyid);
+            skey->rawid = pkey->pubkey->subkeys->keyid;
+        }
         
         /* The location */
         if (pkey->pubkey->keylist_mode & GPGME_KEYLIST_MODE_EXTERN && 
@@ -154,7 +157,13 @@ changed_key (SeahorsePGPKey *pkey)
             skey->location = SKEY_LOC_LOCAL;
         
         /* The type */
-        skey->etype = pkey->seckey ? SKEY_PRIVATE : SKEY_PUBLIC;
+        if (pkey->seckey) {
+            skey->etype = SKEY_PRIVATE;
+            skey->keydesc = _("Public PGP Key");
+        } else {
+            skey->etype = SKEY_PUBLIC;
+            skey->keydesc = _("Private PGP Key");
+        }
 
         /* The info loaded */
         if (pkey->pubkey->keylist_mode & GPGME_KEYLIST_MODE_SIGS && 
@@ -169,7 +178,8 @@ changed_key (SeahorsePGPKey *pkey)
         if (!pkey->pubkey->disabled && !pkey->pubkey->expired && 
             !pkey->pubkey->revoked && !pkey->pubkey->invalid)
         {
-            skey->flags |= SKEY_FLAG_IS_VALID;
+            if (calc_validity (pkey) >= SEAHORSE_VALIDITY_MARGINAL)
+                skey->flags |= SKEY_FLAG_IS_VALID;
             if (pkey->pubkey->can_encrypt)
                 skey->flags |= SKEY_FLAG_CAN_ENCRYPT;
             if (pkey->seckey && pkey->pubkey->can_sign)
@@ -185,7 +195,9 @@ changed_key (SeahorsePGPKey *pkey)
         if (pkey->pubkey->disabled)
             skey->flags |= SKEY_FLAG_DISABLED;
         
-        if (calc_trust (pkey) >= SEAHORSE_VALIDITY_MARGINAL)
+        if (calc_trust (pkey) >= SEAHORSE_VALIDITY_MARGINAL && 
+            !pkey->pubkey->revoked && !pkey->pubkey->disabled && 
+            !pkey->pubkey->expired)
             skey->flags |= SKEY_FLAG_TRUSTED;
     }
     
@@ -237,7 +249,7 @@ seahorse_pgp_key_get_name (SeahorseKey *skey, guint index)
     pkey = SEAHORSE_PGP_KEY (skey);
 
     uid = seahorse_pgp_key_get_nth_userid (pkey, index);
-    return uid ? convert_string (uid->uid) : NULL;
+    return uid ? convert_string (uid->uid, FALSE) : NULL;
 }
 
 static gchar* 
@@ -253,6 +265,58 @@ seahorse_pgp_key_get_name_cn (SeahorseKey *skey, guint index)
     return uid && uid->email ? g_strdup (uid->email) : NULL;
 }
 
+static gchar*
+seahorse_pgp_key_get_name_markup (SeahorseKey *skey, guint index)
+{
+    SeahorsePGPKey *pkey;
+    gpgme_user_id_t uid;
+    gchar *email, *name, *comment, *ret;
+    
+    g_assert (SEAHORSE_IS_PGP_KEY (skey));
+    pkey = SEAHORSE_PGP_KEY (skey);
+
+    uid = seahorse_pgp_key_get_nth_userid (pkey, index);
+    g_return_val_if_fail (uid != NULL, NULL);
+    
+    name = convert_string (uid->name, TRUE);
+    email = convert_string (uid->email, TRUE);
+    comment = convert_string (uid->comment, TRUE);
+    
+    ret = g_strconcat (name, 
+            "<span foreground='#555555' size='small' rise='0'>",
+            email && email[0] ? "  " : "",
+            email && email[0] ? email : "",
+            comment && comment[0] ? "  '" : "",
+            comment && comment[0] ? comment : "",
+            comment && comment[0] ? "'" : "",
+            "</span>", NULL);
+    
+    g_free (name);
+    g_free (comment);
+    g_free (email);
+    
+    return ret;
+}
+
+static SeahorseValidity  
+seahorse_pgp_key_get_name_validity  (SeahorseKey *skey, guint index)
+{
+    SeahorsePGPKey *pkey;
+    gpgme_user_id_t uid;
+
+    g_assert (SEAHORSE_IS_PGP_KEY (skey));
+    pkey = SEAHORSE_PGP_KEY (skey);
+    g_return_val_if_fail (pkey->pubkey, SEAHORSE_VALIDITY_UNKNOWN);
+
+    if (pkey->pubkey->revoked)
+        return SEAHORSE_VALIDITY_REVOKED;
+    if (pkey->pubkey->disabled)
+        return SEAHORSE_VALIDITY_DISABLED;
+    
+    uid = seahorse_pgp_key_get_nth_userid (pkey, index);
+    return uid ? seahorse_validity_from_gpgme (uid->validity) : SEAHORSE_VALIDITY_UNKNOWN;
+}
+
 static void
 seahorse_pgp_key_get_property (GObject *object, guint prop_id,
                                GValue *value, GParamSpec *pspec)
@@ -363,6 +427,8 @@ seahorse_pgp_key_class_init (SeahorsePGPKeyClass *klass)
     key_class->get_num_names = seahorse_pgp_key_get_num_names;
     key_class->get_name = seahorse_pgp_key_get_name;
     key_class->get_name_cn = seahorse_pgp_key_get_name_cn;
+    key_class->get_name_markup = seahorse_pgp_key_get_name_markup;
+    key_class->get_name_validity = seahorse_pgp_key_get_name_validity;
     
     g_object_class_install_property (gobject_class, PROP_PUBKEY,
         g_param_spec_pointer ("pubkey", "Gpgme Public Key", "Gpgme Public Key that this object represents",
@@ -503,7 +569,7 @@ gchar*
 seahorse_pgp_key_get_userid (SeahorsePGPKey *pkey, guint index)
 {
     gpgme_user_id_t uid = seahorse_pgp_key_get_nth_userid (pkey, index);
-    return uid ? convert_string (uid->uid) : NULL;
+    return uid ? convert_string (uid->uid, FALSE) : NULL;
 }
 
 /**
@@ -520,7 +586,7 @@ gchar*
 seahorse_pgp_key_get_userid_name (SeahorsePGPKey *pkey, guint index)
 {
     gpgme_user_id_t uid = seahorse_pgp_key_get_nth_userid (pkey, index);
-    return uid ? convert_string (uid->name) : NULL;
+    return uid ? convert_string (uid->name, FALSE) : NULL;
 }
 
 /**
@@ -537,7 +603,7 @@ gchar*
 seahorse_pgp_key_get_userid_email (SeahorsePGPKey *pkey, guint index)
 {
     gpgme_user_id_t uid = seahorse_pgp_key_get_nth_userid (pkey, index);
-    return uid ? convert_string (uid->email) : NULL;
+    return uid ? convert_string (uid->email, FALSE) : NULL;
 }
 
 /**
@@ -554,7 +620,7 @@ gchar*
 seahorse_pgp_key_get_userid_comment (SeahorsePGPKey *pkey, guint index)
 {
     gpgme_user_id_t uid = seahorse_pgp_key_get_nth_userid (pkey, index);
-    return uid ? convert_string (uid->comment) : NULL;
+    return uid ? convert_string (uid->comment, FALSE) : NULL;
 }
 
 guint           
@@ -745,11 +811,11 @@ seahorse_pgp_key_get_signature_text (SeahorsePGPKey *pkey, gpgme_key_sig_t signa
     g_return_if_fail (signature != NULL);
     
     if (name)
-        *name = signature->name ? convert_string (signature->name) : NULL;
+        *name = signature->name ? convert_string (signature->name, FALSE) : NULL;
     if (email)
-        *email = signature->email ? convert_string (signature->email) : NULL;
+        *email = signature->email ? convert_string (signature->email, FALSE) : NULL;
     if (comment)
-        *comment = signature->comment ? convert_string (signature->comment) : NULL;
+        *comment = signature->comment ? convert_string (signature->comment, FALSE) : NULL;
 }
 
 guint         
